"""Factories for the review app."""
import factory

from django_libs.tests.factories import HvadFactoryMixin, UserFactory

from .. import models


class ReviewFactory(factory.DjangoModelFactory):
    """Factory for the ``Review`` model."""
    FACTORY_FOR = models.Review

    reviewed_item = factory.SubFactory(UserFactory)


class ReviewExtraInfoFactory(factory.DjangoModelFactory):
    """Factory for the ``ReviewExtraInfo`` model."""
    FACTORY_FOR = models.ReviewExtraInfo

    review = factory.SubFactory(ReviewFactory)
    type = 'extra_information'
    content_object = factory.SubFactory(UserFactory)


class RatingCategoryFactory(HvadFactoryMixin, factory.DjangoModelFactory):
    """Factory for the ``RatingCategory`` model."""
    FACTORY_FOR = models.RatingCategory

    language_code = 'en'
    name = factory.Sequence(lambda x: 'Rating category {}'.format(x))


class RatingCategoryChoiceFactory(HvadFactoryMixin,
                                  factory.DjangoModelFactory):
    """Factory for the ``RatingCategoryChoice`` model."""
    FACTORY_FOR = models.RatingCategoryChoice

    ratingcategory = factory.SubFactory(RatingCategoryFactory)
    label = factory.Sequence(lambda n: 'label {0}'.format(n))
    value = factory.Sequence(lambda n: str(n))


class RatingFactory(factory.DjangoModelFactory):
    """Factory for the ``Rating`` model."""
    FACTORY_FOR = models.Rating

    value = '3'
    review = factory.SubFactory(ReviewFactory)
    category = factory.SubFactory(RatingCategoryFactory)



"""URLs to run the tests."""
from django.conf.urls import patterns, include, url
from django.contrib import admin
from django.views.generic import ListView

from ..models import Review


admin.autodiscover()

urlpatterns = patterns(
    '',
    url(r'^review-listing/', ListView.as_view(model=Review),
        name='review_list'),
    url(r'^admin/', include(admin.site.urls)),
    url(r'^umedia/', include('user_media.urls')),
    url(r'^review/', include('review.urls')),
)



"""Template tags for the ``review`` app."""
from django.contrib.contenttypes.models import ContentType
from django.template import Library

from .. import models


register = Library()


@register.assignment_tag
def total_review_average(obj, normalize_to=100):
    """Returns the average for all reviews of the given object."""
    ctype = ContentType.objects.get_for_model(obj.__class__)
    total_average = 0
    reviews = models.Review.objects.filter(
        content_type=ctype, object_id=obj.id)
    for review in reviews:
        total_average += review.get_average_rating(normalize_to)
    if reviews:
        total_average /= reviews.count()
    return total_average


@register.assignment_tag
def user_has_reviewed(obj, user):
    """Returns True if the user has already reviewed the object."""
    ctype = ContentType.objects.get_for_model(obj.__class__)
    try:
        models.Review.objects.get(user=user, content_type=ctype,
                                  object_id=obj.id)
    except models.Review.DoesNotExist:
        return False
    return True



"""Forms for the ``test_app`` app."""
from django import forms

from review.forms import ReviewForm
from review.models import ReviewExtraInfo
from .models import WeatherCondition


class CustomReviewForm(ReviewForm):
    """Form to make use of the review extra info."""
    def __init__(self, reviewed_item, user=None, *args, **kwargs):
        super(CustomReviewForm, self).__init__(
            reviewed_item, user, *args, **kwargs)
        choices = [(x.pk, x.name) for x in WeatherCondition.objects.all()]
        self.fields['weather_conditions'] = forms.fields.ChoiceField(
            choices=choices)

    def save(self, *args, **kwargs):
        self.instance = super(CustomReviewForm, self).save(*args, **kwargs)
        ReviewExtraInfo.objects.create(
            type='weather_conditions',
            review=self.instance,
            content_object=WeatherCondition.objects.get(
                pk=self.cleaned_data['weather_conditions']),
        )
        return self.instance



"""Test for the template tags of the ``review`` app."""
from django.test import TestCase

from django_libs.tests.factories import UserFactory

from ..templatetags.review_tags import total_review_average, user_has_reviewed
from . import factories


class TotalReviewAverageTestCase(TestCase):
    """Tests for the ``total_review_average`` template tag."""
    longMessage = True

    def setUp(self):
        self.content_object = UserFactory()
        self.review = factories.ReviewFactory(
            reviewed_item=self.content_object)
        self.rating1 = factories.RatingFactory(review=self.review, value='4')
        # we create choices to simulate, that the previous value was the max
        for i in range(0, 5):
            factories.RatingCategoryChoiceFactory(
                ratingcategory=self.rating1.category, value=i)
        self.rating2 = factories.RatingFactory(review=self.review, value='6')
        # we create choices to simulate, that the previous value was the max
        for i in range(0, 7):
            factories.RatingCategoryChoiceFactory(
                ratingcategory=self.rating2.category, value=i)

    def test_tag(self):
        self.assertEqual(total_review_average(self.content_object), 100)
        factories.RatingFactory(
            category=self.rating1.category,
            review=self.review, value='0')
        factories.RatingFactory(
            category=self.rating2.category,
            review=self.review, value='0')
        self.assertEqual(total_review_average(self.content_object), 50)
        factories.RatingFactory(
            category=self.rating1.category,
            review=self.review, value=None)
        factories.RatingFactory(
            category=self.rating2.category,
            review=self.review, value=None)
        self.assertEqual(total_review_average(self.content_object), 50)
        self.assertEqual(total_review_average(self.content_object, 10), 5)
        self.assertEqual(total_review_average(self.content_object, 5), 2.5)


class UserHasReviewedTestCase(TestCase):
    """Tests for the ``user_has_reviewed`` template tag."""
    longMessage = True

    def setUp(self):
        self.user = UserFactory()
        self.content_object = UserFactory()
        self.review = factories.ReviewFactory(
            user=self.user,
            reviewed_item=self.content_object)
        self.other_user = UserFactory()

    def test_tag(self):
        self.assertTrue(user_has_reviewed(self.content_object, self.user))
        self.assertFalse(user_has_reviewed(self.content_object,
                                           self.other_user))



"""URLs for the review app."""
from django.conf.urls import patterns, url

from . import views


urlpatterns = patterns(
    '',
    url(r'^(?P<pk>\d+)/delete/$',
        views.ReviewDeleteView.as_view(),
        name='review_delete'),
    url(r'^(?P<pk>\d+)/update/$',
        views.ReviewUpdateView.as_view(),
        name='review_update'),
    url(r'^(?P<pk>\d+)/$',
        views.ReviewDetailView.as_view(),
        name='review_detail'),
    url(r'^(?P<content_type>[-\w]+)/(?P<object_id>\d+)/create/$',
        views.ReviewCreateView.as_view(),
        name='review_create'),
)



"""Just an empty models file to let the testrunner recognize this as app."""
from django.conf import settings
from django.contrib.contenttypes import generic
from django.contrib.contenttypes.models import ContentType
from django.db import models
from django.utils import timezone
from django.utils.translation import ugettext, ugettext_lazy as _

from hvad.models import TranslatableModel, TranslatedFields

DEFAULT_CHOICES = (
    ('1', '1'),
    ('2', '2'),
    ('3', '3'),
    ('4', '4'),
    ('5', '5'),
)


class Review(models.Model):
    """
    Represents a user review, which includes free text and images.

    :reviewed_item: Object, which is reviewed.
    :user (optional): User, which posted the rating.
    :content (optional): Running text.
    :images (optional): Review-related images.
    :language (optional): Language shortcut to filter reviews.
    :creation_date: The date and time, this review was created.
    :average_rating: Should always be calculated and updated when the object is
      saved. This is for improving performance and reducing db queries when
      calculating ratings for reviewed items. Currently it gets updated at the
      end of the save method of the ``ReviewForm``. This means that when you
      manually save a Review via the Django admin, this field will not be
      updated.
    :extra_item: Optional object, which should be attached to the review.

    """
    # GFK 'reviewed_item'
    content_type = models.ForeignKey(ContentType)
    object_id = models.PositiveIntegerField()
    reviewed_item = generic.GenericForeignKey('content_type', 'object_id')

    user = models.ForeignKey(
        'auth.User',
        verbose_name=_('User'),
        blank=True, null=True,
    )

    content = models.TextField(
        max_length=1024,
        verbose_name=_('Content'),
        blank=True,
    )

    images = generic.GenericRelation(
        'user_media.UserMediaImage',
    )

    language = models.CharField(
        max_length=5,
        verbose_name=_('Language'),
        blank=True,
    )

    creation_date = models.DateTimeField(
        auto_now_add=True,
        verbose_name=_('Creation date'),
    )

    average_rating = models.FloatField(
        verbose_name=_('Average rating'),
        default=0,
    )

    # GFK 'extra_item'
    extra_content_type = models.ForeignKey(
        ContentType,
        related_name='reviews_attached',
        null=True, blank=True,
    )
    extra_object_id = models.PositiveIntegerField(null=True, blank=True)
    extra_item = generic.GenericForeignKey(
        'extra_content_type', 'extra_object_id')

    class Meta:
        ordering = ['-creation_date']

    def __unicode__(self):
        return '{0} - {1}'.format(self.reviewed_item, self.get_user())

    # TODO: Add magic to get ReviewExtraInfo content objects here

    def get_user(self):
        """Returns the user who wrote this review or ``Anonymous``."""
        if self.user:
            return self.user.email
        return ugettext('Anonymous')

    def get_average_rating(self, max_value=None):
        """
        Returns the average rating for all categories of this review.

        This is useful if you have several ratings for different categories
        like ``Service``, ``Product Quality`` and want to show a total rating
        for this review.

        :param max_value: By default the app is set to a rating from 1 to 5.
          So if nothing is changed, we can just calculate the average of all
          rating values and be good. We then have an average that is between 1
          and 5 as well.
          BUT if  we have custom choices, we could end up having one category
          with a range of 1 to 10 and one category with 1 to 5. The result then
          must be abstracted to fit into the given range set by max_value.

          This can also be used to calculate percentages by setting max_value
          to 100.

        """
        max_rating_value = 0
        category_maximums = {}
        category_averages = []
        categories = RatingCategory.objects.filter(counts_for_average=True,
                                                   rating__review=self)
        # find the highest rating possible across all categories
        for category in categories:
            category_max = category.get_rating_max_from_choices()
            category_maximums.update({category: category_max})
            if max_value is not None:
                max_rating_value = max_value
            else:
                if category_max > max_rating_value:
                    max_rating_value = category_max
        # calculate the average of every distinct category, normalized to the
        # recently found max
        for category in categories:
            category_average = 0.0
            ratings = Rating.objects.filter(
                category=category, value__isnull=False).exclude(value='')
            category_max = category_maximums[category]
            for rating in ratings:
                category_average += float(rating.value)

            category_average *= float(max_rating_value) / float(category_max)
            if category_average:
                category_averages.append(category_average / ratings.count())

        # calculate the total average of all categories
        total_average = 0
        for category_average in category_averages:
            total_average += category_average
        if not len(category_averages):
            return False
        total_average /= len(category_averages)

        return total_average

    def is_editable(self):
        """
        Returns True, if the time period to update this review hasn't ended
        yet.

        If the period setting has not been set, it always return True. This
        is the general case. If the user has used this setting to define an
        update period it returns False, if this period has expired.

        """
        if getattr(settings, 'REVIEW_UPDATE_PERIOD', False):
            period_end = self.creation_date + timezone.timedelta(
                seconds=getattr(settings, 'REVIEW_UPDATE_PERIOD') * 60)
            if timezone.now() > period_end:
                return False
        return True


class ReviewExtraInfo(models.Model):
    """
    Model to add any extra information to a review.

    This can be useful if you need to save more information about a reviewer
    than just the User instance. Let's say you are building a site for theme
    park reviews and you want to allow the user to select the weather
    conditions for the day of his visit (which will surely influence his
    review). This model would allow you to tie any model of your app to a
    review.

    :type: Callable type of the extra info. This should be unique per review.
      We will soon add a hack to the Review model which allows you to get the
      content_object of this instance from a review instance (i.e. by calling
      ``my_review.weather_conditions.name``). So for this example you would
      set the type to ``weather_conditions``.
    :review: Related review.
    :content_object: The related object that stores this extra information.

    """
    type = models.CharField(
        max_length=256,
        verbose_name=_('Type'),
    )

    review = models.ForeignKey(
        'review.Review',
        verbose_name=_('Review'),
    )

    # GFK 'content_object'
    content_type = models.ForeignKey(ContentType)
    object_id = models.PositiveIntegerField()
    content_object = generic.GenericForeignKey('content_type', 'object_id')

    class Meta:
        ordering = ['type']

    def __unicode__(self):
        return '{0} - {1}'.format(self.review, self.type)


class RatingCategory(TranslatableModel):
    """
    Represents a rating category.

    If your reviews are just text based, you don't have to use this.

    This can be useful if you want to allow users to rate one or more
    categories, like ``Food``, ``Room service``, ``Cleansines`` and so on.

    :identifier: Optional identifier.
    :name: Name of the category.
    :counts_for_average: If True, the ratings of this category will be used to
      calculate the average rating. Default is True.

    """
    identifier = models.SlugField(
        max_length=32,
        verbose_name=_('Identifier'),
        blank=True,
    )

    counts_for_average = models.BooleanField(
        verbose_name=_('Counts for average rating'),
        default=True,
    )

    translations = TranslatedFields(
        name=models.CharField(max_length=256),
    )

    def __unicode__(self):
        return self.lazy_translation_getter('name', 'Untranslated')

    @property
    def required(self):
        """Returns False, if the choices include a None value."""
        if not hasattr(self, '_required'):
            # get_choices sets _required
            self.get_choices()
        return self._required

    def get_choices(self):
        """Returns the tuple of choices for this category."""
        choices = ()
        self._required = True
        for choice in self.choices.all():
            if choice.value is None or choice.value == '':
                self._required = False
            choices += (choice.value, choice.label),
        if not choices:
            return DEFAULT_CHOICES
        return choices

    def get_rating_max_from_choices(self):
        """Returns the maximun value a rating can have in this catgory."""
        return int(list(self.get_choices())[0][0])


class RatingCategoryChoice(TranslatableModel):
    """
    Defines an optional choice for a `RatingCategory`.

    If `RatingChoice` exists, the choices will not be loaded from the settings.

    :label: The label that is displayed for this choice.
    :ratingcategory: The `RatingCategory` this choice belongs to.
    :value: The value that this choice has. If a `RatingChoice` with value=None
      is created and chosen by the user, this category is not taken into
      account when the average is calculated.

    """
    ratingcategory = models.ForeignKey(
        RatingCategory,
        verbose_name=_('Rating category'),
        related_name='choices',
    )

    value = models.CharField(
        verbose_name=_('Value'),
        max_length=20,
        blank=True, null=True,
    )

    translations = TranslatedFields(
        label=models.CharField(
            verbose_name=_('Label'),
            max_length=128,
        ),
    )

    def __unicode__(self):
        return self.lazy_translation_getter('label',
                                            self.ratingcategory.identifier)

    class Meta:
        ordering = ('-value', )


class Rating(models.Model):
    """
    Represents a rating for one rating category.

    :rating: Rating value.
    :review: The review the rating belongs to.
    :category: The rating category the rating belongs to.

    """
    rating_choices = DEFAULT_CHOICES

    value = models.CharField(
        max_length=20,
        verbose_name=_('Value'),
        choices=getattr(settings, 'REVIEW_RATING_CHOICES', rating_choices),
        blank=True, null=True,
    )

    review = models.ForeignKey(
        'review.Review',
        verbose_name=_('Review'),
        related_name='ratings',
    )

    category = models.ForeignKey(
        'review.RatingCategory',
        verbose_name=_('Category'),
    )

    class Meta:
        ordering = ['category', 'review']

    def __unicode__(self):
        return '{0}/{1} - {2}'.format(self.category, self.review, self.value)



# flake8: noqa
# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding model 'Review'
        db.create_table(u'review_review', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('content_type', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['contenttypes.ContentType'])),
            ('object_id', self.gf('django.db.models.fields.PositiveIntegerField')()),
            ('user', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['auth.User'], null=True, blank=True)),
            ('content', self.gf('django.db.models.fields.TextField')(max_length=1024, blank=True)),
            ('language', self.gf('django.db.models.fields.CharField')(max_length=5, blank=True)),
            ('creation_date', self.gf('django.db.models.fields.DateTimeField')(auto_now_add=True, blank=True)),
        ))
        db.send_create_signal(u'review', ['Review'])

        # Adding model 'ReviewExtraInfo'
        db.create_table(u'review_reviewextrainfo', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('type', self.gf('django.db.models.fields.CharField')(max_length=256)),
            ('review', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['review.Review'])),
            ('content_type', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['contenttypes.ContentType'])),
            ('object_id', self.gf('django.db.models.fields.PositiveIntegerField')()),
        ))
        db.send_create_signal(u'review', ['ReviewExtraInfo'])

        # Adding model 'RatingCategory'
        db.create_table(u'review_ratingcategory', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
        ))
        db.send_create_signal(u'review', ['RatingCategory'])

        # Adding model 'RatingCategoryTranslation'
        db.create_table(u'review_ratingcategorytranslation', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('name', self.gf('django.db.models.fields.CharField')(max_length=256)),
            ('category', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['review.RatingCategory'])),
            ('language', self.gf('django.db.models.fields.CharField')(max_length=2)),
        ))
        db.send_create_signal(u'review', ['RatingCategoryTranslation'])

        # Adding model 'Rating'
        db.create_table(u'review_rating', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('value', self.gf('django.db.models.fields.CharField')(max_length=20)),
            ('review', self.gf('django.db.models.fields.related.ForeignKey')(related_name='ratings', to=orm['review.Review'])),
            ('category', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['review.RatingCategory'])),
        ))
        db.send_create_signal(u'review', ['Rating'])


    def backwards(self, orm):
        # Deleting model 'Review'
        db.delete_table(u'review_review')

        # Deleting model 'ReviewExtraInfo'
        db.delete_table(u'review_reviewextrainfo')

        # Deleting model 'RatingCategory'
        db.delete_table(u'review_ratingcategory')

        # Deleting model 'RatingCategoryTranslation'
        db.delete_table(u'review_ratingcategorytranslation')

        # Deleting model 'Rating'
        db.delete_table(u'review_rating')


    models = {
        u'auth.group': {
            'Meta': {'object_name': 'Group'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '80'}),
            'permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': u"orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'})
        },
        u'auth.permission': {
            'Meta': {'ordering': "(u'content_type__app_label', u'content_type__model', u'codename')", 'unique_together': "((u'content_type', u'codename'),)", 'object_name': 'Permission'},
            'codename': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['contenttypes.ContentType']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        u'auth.user': {
            'Meta': {'object_name': 'User'},
            'date_joined': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75', 'blank': 'True'}),
            'first_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'groups': ('django.db.models.fields.related.ManyToManyField', [], {'to': u"orm['auth.Group']", 'symmetrical': 'False', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'is_staff': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_superuser': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'last_login': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'last_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'password': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'user_permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': u"orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'}),
            'username': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '30'})
        },
        u'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        u'review.rating': {
            'Meta': {'ordering': "['category', 'review']", 'object_name': 'Rating'},
            'category': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['review.RatingCategory']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'review': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'ratings'", 'to': u"orm['review.Review']"}),
            'value': ('django.db.models.fields.CharField', [], {'max_length': '20'})
        },
        u'review.ratingcategory': {
            'Meta': {'object_name': 'RatingCategory'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'})
        },
        u'review.ratingcategorytranslation': {
            'Meta': {'object_name': 'RatingCategoryTranslation'},
            'category': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['review.RatingCategory']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'language': ('django.db.models.fields.CharField', [], {'max_length': '2'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '256'})
        },
        u'review.review': {
            'Meta': {'ordering': "['-creation_date']", 'object_name': 'Review'},
            'content': ('django.db.models.fields.TextField', [], {'max_length': '1024', 'blank': 'True'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['contenttypes.ContentType']"}),
            'creation_date': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'language': ('django.db.models.fields.CharField', [], {'max_length': '5', 'blank': 'True'}),
            'object_id': ('django.db.models.fields.PositiveIntegerField', [], {}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['auth.User']", 'null': 'True', 'blank': 'True'})
        },
        u'review.reviewextrainfo': {
            'Meta': {'ordering': "['type']", 'object_name': 'ReviewExtraInfo'},
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['contenttypes.ContentType']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'object_id': ('django.db.models.fields.PositiveIntegerField', [], {}),
            'review': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['review.Review']"}),
            'type': ('django.db.models.fields.CharField', [], {'max_length': '256'})
        },
        u'user_media.usermediaimage': {
            'Meta': {'object_name': 'UserMediaImage'},
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['contenttypes.ContentType']", 'null': 'True', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'image': ('django.db.models.fields.files.ImageField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'object_id': ('django.db.models.fields.PositiveIntegerField', [], {'null': 'True', 'blank': 'True'}),
            'position': ('django.db.models.fields.PositiveIntegerField', [], {'null': 'True', 'blank': 'True'}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['auth.User']"})
        }
    }

    complete_apps = ['review']


"""View tests for the ``review`` app."""
from django.contrib.contenttypes.models import ContentType
from django.core.urlresolvers import reverse
from django.test import TestCase
from django.utils import timezone

from django_libs.tests.factories import UserFactory
from django_libs.tests.mixins import ViewTestMixin

from .factories import ReviewFactory
from ..models import Review, ReviewExtraInfo
from test_app.factories import WeatherConditionFactory


class ReviewCreateViewTestCase(ViewTestMixin, TestCase):
    def setUp(self):
        self.content_object = UserFactory()
        self.user = UserFactory()

    def get_view_name(self):
        return 'review_create'

    def get_view_kwargs(self):
        return {
            'content_type': ContentType.objects.get_for_model(
                self.content_object),
            'object_id': self.content_object.pk,
        }

    def user_has_perm(self, user):
        # Simulates permission for the current user
        if user.first_name:
            return True
        return False

    def test_view(self):
        wrong_kwargs = {
            'content_type': 'Foo',
            'object_id': self.content_object.pk,
        }
        self.is_not_callable(kwargs=wrong_kwargs)

        wrong_kwargs = {
            'content_type': ContentType.objects.get_for_model(
                self.content_object),
            'object_id': '999',
        }
        self.is_not_callable(kwargs=wrong_kwargs)

        self.is_callable(user=self.user, message=(
            'View should be callable by an authenticated user.'))

        with self.settings(REVIEW_AVOID_MULTIPLE_REVIEWS=True):
            has_perm = lambda u, item: self.user_has_perm(u)
            with self.settings(REVIEW_PERMISSION_FUNCTION=has_perm):
                self.is_not_callable(method='post', user=self.user, message=(
                    'View should not be callable due to missing permissions.'))
                self.assertEqual(Review.objects.count(), 0, msg=(
                    'No reviews should\'ve been created.'))
                self.user.first_name = 'Foo'
                self.user.save()
                self.is_callable(method='post', user=self.user, message=(
                    'View should be callable by an authenticated user.'))
                self.assertEqual(Review.objects.count(), 1, msg=(
                    'One review should\'ve been created.'))
            self.is_callable(
                and_redirects_to=reverse('review_update', kwargs={
                    'pk': Review.objects.all()[0].pk}),
                method='post', user=self.user, message=(
                    'View should redirect, if review alreasy exists.'))
            self.assertEqual(Review.objects.count(), 1, msg=(
                'No new review should\'ve been created.'))

        with self.settings(REVIEW_CUSTOM_FORM='test_app.FooReviewForm'):
            self.is_callable()

        with self.settings(REVIEW_CUSTOM_FORM='foo.BarForm'):
            self.is_callable()

        with self.settings(
                REVIEW_CUSTOM_FORM='test_app.forms.CustomReviewForm'):
            data = {'weather_conditions': WeatherConditionFactory().pk}
            self.is_callable(method='post', data=data)
            self.assertEqual(ReviewExtraInfo.objects.count(), 1, msg=(
                'One review extra info should\'ve been created.'))

        with self.settings(REVIEW_UPDATE_SUCCESS_URL='review_list'):
            self.is_callable(method='post', data=data,
                             and_redirects_to=reverse('review_list'))

        with self.settings(REVIEW_ALLOW_ANONYMOUS=True):
            self.is_callable(anonymous=True, message=(
                'View should be callable as an anonymous user.'))


class ReviewDetailViewTestCase(ViewTestMixin, TestCase):
    def setUp(self):
        self.review = ReviewFactory()

    def get_view_name(self):
        return 'review_detail'

    def get_view_kwargs(self):
        return {'pk': self.review.pk}

    def test_view(self):
        self.is_callable()


class ReviewUpdateViewTestCase(ViewTestMixin, TestCase):
    def setUp(self):
        self.user = UserFactory()
        self.other_user = UserFactory()
        self.review = ReviewFactory(user=self.user)

    def get_view_name(self):
        return 'review_update'

    def get_view_kwargs(self):
        return {'pk': self.review.pk}

    def test_view(self):
        self.is_not_callable(user=self.other_user)
        self.is_callable(user=self.user)

        with self.settings(REVIEW_UPDATE_PERIOD=1):
            self.is_callable(message=(
                'Should be callable, if period hasn\'t ended yet.'))
            self.review.creation_date = timezone.now() - timezone.timedelta(
                seconds=120)
            self.review.save()
            self.is_callable(
                and_redirects_to=reverse('review_detail', kwargs={
                    'pk': self.review.pk}),
                message=('Should redirect, if period has ended.'))


class ReviewDeleteViewTestCase(ViewTestMixin, TestCase):
    def setUp(self):
        self.user = UserFactory()
        self.review = ReviewFactory(user=self.user)

    def get_view_name(self):
        return 'review_delete'

    def get_view_kwargs(self):
        return {'pk': self.review.pk}

    def test_view(self):
        self.is_callable(user=self.user, method='post')
        self.review = ReviewFactory(user=self.user)
        with self.settings(REVIEW_DELETION_SUCCESS_URL='review_list'):
            self.is_callable(user=self.user, method='post')



"""Form tests for the ``review`` app."""
from django.test import TestCase

from django_libs.tests.factories import UserFactory

from .factories import RatingCategoryFactory, RatingCategoryChoiceFactory
from ..forms import ReviewForm
from ..models import Review, Rating


class ReviewFormTestCase(TestCase):
    longMessage = True

    def setUp(self):
        self.user = UserFactory()
        self.content_object = UserFactory()
        self.rating_category = RatingCategoryFactory()

    def test_form(self):
        form = ReviewForm(reviewed_item=self.content_object)
        self.assertTrue(form, msg=('Form has been initiated.'))

        data = {'category_{0}'.format(self.rating_category.pk): '3'}
        form = ReviewForm(reviewed_item=self.content_object, data=data)
        self.assertTrue(form.is_valid(), msg=('Form should be valid.'))

        review = form.save()
        self.assertEqual(Review.objects.count(), 1, msg=(
            'One review should have been created.'))
        self.assertEqual(
            Rating.objects.all()[0].review,
            Review.objects.all()[0],
            msg=('The rating\'s review should be equal the form\'s instance.'))
        self.assertEqual(
            Rating.objects.all()[0].category.pk,
            self.rating_category.pk,
            msg=('The rating\'s category should be saved.'))
        self.assertEqual(Rating.objects.all()[0].value, '3', msg=(
            'The rating\'s value should be saved.'))
        self.assertIsNone(review.user, msg=('User should be None.'))

        form = ReviewForm(user=self.user, reviewed_item=self.content_object,
                          data=data)
        self.assertTrue(form.is_valid(), msg=('Form should be valid.'))

        review = form.save()
        self.assertEqual(Review.objects.count(), 2, msg=(
            'Another review should have been created.'))
        self.assertIsNotNone(review.user, msg=('User should be existant.'))

        self.new_category = RatingCategoryFactory()
        form = ReviewForm(instance=review, reviewed_item=self.content_object)
        self.assertEqual(
            form.initial.get('category_{0}'.format(self.rating_category.pk)),
            '3', msg=('The form\'s initial should contain the ratings.'))
        self.assertFalse(
            form.initial.get('category_{0}'.format(self.new_category.pk)),
            msg=('The form\'s initial should not contain a new category.'))

    def test_form_with_custom_choices(self):
        # Create custom choices
        choices = []
        expected_choices = []
        for j in range(1, 4):
            i = 5 - j
            choices.append(RatingCategoryChoiceFactory(
                ratingcategory=self.rating_category,
                value=i, label=str(i)))
            expected_choices.append((unicode(i), unicode(i)))

        form = ReviewForm(reviewed_item=self.content_object)
        self.assertTrue(form, msg=('Form has been initiated.'))
        field_name = 'category_{0}'.format(self.rating_category.pk)
        self.assertTrue(form[field_name], msg='The field was added')
        self.assertEqual(form[field_name].field.choices, expected_choices,
                         msg=('The field choices were not added correctly from'
                              ' the RatingCategegoryChoice instances.'))

        data = {'category_{0}'.format(self.rating_category.pk): '5'}
        form = ReviewForm(reviewed_item=self.content_object, data=data)
        self.assertFalse(form.is_valid(), msg=(
            'When assigning a higher value, than there are'
            ' RatingCategoryChoice objects, the form should not be valid.'))

        data = {'category_{0}'.format(self.rating_category.pk): '3'}
        form = ReviewForm(reviewed_item=self.content_object, data=data)
        self.assertTrue(form.is_valid(), msg=(
            'Even when answering without value and nullifying the category,'
            ' the form should be valid. Errors: {0}'.format(form.errors)))

        data = {'category_{0}'.format(self.rating_category.pk): '3'}
        form = ReviewForm(reviewed_item=self.content_object, data=data)
        self.assertTrue(form.is_valid(), msg=(
            'The form should be valid. Errors: {0}'.format(form.errors)))

        review = form.save()
        self.assertEqual(Review.objects.count(), 1, msg=(
            'One review should have been created.'))
        self.assertEqual(
            Rating.objects.all()[0].review,
            Review.objects.all()[0],
            msg=('The rating\'s review should be equal the form\'s instance.'))
        self.assertEqual(
            Rating.objects.all()[0].category.pk,
            self.rating_category.pk,
            msg=('The rating\'s category should be saved.'))
        self.assertEqual(Rating.objects.all()[0].value, '3', msg=(
            'The rating\'s value should be saved.'))
        self.assertIsNone(review.user, msg=('User should be None.'))



"""Factories for the test_app app."""
import factory

from . import models


class WeatherConditionFactory(factory.DjangoModelFactory):
    """Factory for the ``WeatherCondition`` model."""
    FACTORY_FOR = models.WeatherCondition

    name = 'Good'



"""Forms for the ``review`` app."""
from django import forms
from django.conf import settings
from django.utils.translation import get_language

from .models import Review, Rating, RatingCategory


class ReviewForm(forms.ModelForm):
    def __init__(self, reviewed_item, user=None, *args, **kwargs):
        self.user = user
        self.reviewed_item = reviewed_item
        self.widget = getattr(settings, 'REVIEW_FORM_CHOICE_WIDGET', None)
        super(ReviewForm, self).__init__(*args, **kwargs)
        # Dynamically add fields for each rating category
        for category in RatingCategory.objects.all():
            field_name = 'category_{0}'.format(category.pk)
            choices = category.get_choices()
            self.fields[field_name] = forms.ChoiceField(
                choices=choices, label=category.__unicode__(),
            )
            self.fields[field_name].required = category.required
            if self.widget is not None:
                self.fields[field_name].widget = self.widget()
            if self.instance.pk:
                try:
                    self.initial.update({
                        'category_{0}'.format(category.pk): Rating.objects.get(
                            review=self.instance, category=category).value,
                    })
                except Rating.DoesNotExist:
                    pass

    def save(self, *args, **kwargs):
        if not self.instance.pk:
            self.instance.user = self.user
            self.instance.reviewed_item = self.reviewed_item
            self.instance.language = get_language()
        self.instance = super(ReviewForm, self).save(*args, **kwargs)
        # Update or create ratings
        for field in self.fields:
            if field.startswith('category_'):
                rating, created = Rating.objects.get_or_create(
                    review=self.instance,
                    category=RatingCategory.objects.get(
                        pk=field.replace('category_', '')),
                )
                rating.value = self.cleaned_data[field]
                rating.save()

        self.instance.average_rating = self.instance.get_average_rating()
        self.instance.save()
        return self.instance

    class Meta:
        model = Review
        fields = ('content', )



"""Tests for the models of the review app."""
from django.test import TestCase
from django.utils.timezone import now, timedelta

from django_libs.tests.factories import UserFactory

from . import factories


class ReviewTestCase(TestCase):
    longMessage = True

    def setUp(self):
        self.review = factories.ReviewFactory()

    def test_instance(self):
        self.assertTrue(self.review.pk, msg=(
            'Review model should have been created.'))

    def test_get_user(self):
        self.assertEqual(self.review.get_user(), 'Anonymous', msg=(
            'Should return anonymous.'))
        self.user = UserFactory()
        self.review.user = self.user
        self.assertEqual(self.review.get_user(), self.user.email, msg=(
            'Should return a user\'s email.'))

    def test_get_average_rating(self):
        self.assertFalse(self.review.get_average_rating(), msg=(
            'If there are no ratings, it should return False.'))
        factories.RatingFactory(review=self.review, value='2')
        factories.RatingFactory(review=self.review, value='4')
        self.assertEqual(self.review.get_average_rating(), 3, msg=(
            'Should return the average rating value.'))

        factories.RatingFactory(review=self.review, value=None)
        factories.RatingFactory(category__counts_for_average=False,
                                review=self.review, value=0.0)
        self.assertEqual(self.review.get_average_rating(), 3, msg=(
            'Should return the average rating value and exclude the nullified'
            ' ones.'))

    def test_get_average_rating_with_custom_choices(self):
        self.assertFalse(self.review.get_average_rating(), msg=(
            'If there are no ratings, it should return False.'))
        rating1 = factories.RatingFactory(review=self.review, value='4')
        # we create choices to simulate, that the previous value was the max
        for i in range(0, 5):
            factories.RatingCategoryChoiceFactory(
                ratingcategory=rating1.category, value=i)
        rating2 = factories.RatingFactory(review=self.review, value='6')
        # we create choices to simulate, that the previous value was the max
        for i in range(0, 7):
            factories.RatingCategoryChoiceFactory(
                ratingcategory=rating2.category, value=i)
        # testing the absolute max voting
        self.assertEqual(self.review.get_average_rating(6), 6, msg=(
            'Should return the average rating value.'))
        self.assertEqual(self.review.get_average_rating(4), 4, msg=(
            'Should return the average rating value.'))
        self.assertEqual(self.review.get_average_rating(100), 100, msg=(
            'Should return the average rating value.'))

        # these ratings should not change results and should just be ignored
        factories.RatingFactory(
            category=rating2.category, review=self.review, value=None)
        factories.RatingFactory(review=self.review, value=None)
        self.assertEqual(self.review.get_average_rating(6), 6, msg=(
            'Should return the average rating value.'))
        self.assertEqual(self.review.get_average_rating(4), 4, msg=(
            'Should return the average rating value.'))
        self.assertEqual(self.review.get_average_rating(100), 100, msg=(
            'Should return the average rating value.'))

        # altering the ratings to get a very low voting
        rating1.value = '1'
        rating1.save()
        rating2.value = '1'
        rating2.save()
        self.assertEqual(self.review.get_average_rating(6), 1.1666666666666667,
                         msg=('Should return the average rating value.'))
        self.assertEqual(self.review.get_average_rating(4), 0.7777777777777777,
                         msg=('Should return the average rating value.'))
        self.assertEqual(
            self.review.get_average_rating(100),
            19.444444444444446, msg=(
                'Should return the average rating value.'))

        # and finally the lowest possible voting
        rating1.value = '0'
        rating1.save()
        rating2.value = '0'
        rating2.save()
        self.assertEqual(self.review.get_average_rating(6), 0,
                         msg=('Should return the average rating value.'))
        self.assertEqual(self.review.get_average_rating(4), 0,
                         msg=('Should return the average rating value.'))
        self.assertEqual(self.review.get_average_rating(100), 0,
                         msg=('Should return the average rating value.'))

    def test_is_editable(self):
        self.assertTrue(self.review.is_editable(), msg=(
            'Should be editable, if period setting is not set.'))
        with self.settings(REVIEW_UPDATE_PERIOD=1):
            self.assertTrue(self.review.is_editable(), msg=(
                'Should be editable, if period has not ended yet.'))
            self.review.creation_date = now() - timedelta(days=1)
            self.review.save()
            self.assertFalse(self.review.is_editable(), msg=(
                'Should return False, if period has ended.'))


class ReviewExtraInfoTestCase(TestCase):
    longMessage = True

    def setUp(self):
        self.extra_info = factories.ReviewExtraInfoFactory()

    def test_instance(self):
        self.assertTrue(self.extra_info.pk, msg=(
            'Review extra info model should have been created.'))


class RatingCategoryTestCase(TestCase):
    longMessage = True

    def setUp(self):
        self.category = factories.RatingCategoryFactory()

    def test_instance(self):
        self.assertTrue(self.category.pk, msg=(
            'Rating category model should have been created.'))


class RatingCategoryChoiceTestCase(TestCase):
    """Tests for the ``RatingCategoryChoice`` model class."""
    longMessage = True

    def test_instantiation(self):
        """Test instantiation of the ``RatingCategoryChoice`` model."""
        ratingcategorychoice = factories.RatingCategoryChoiceFactory()
        self.assertTrue(ratingcategorychoice.pk)


class RatingTestCase(TestCase):
    longMessage = True

    def setUp(self):
        self.rating = factories.RatingFactory()

    def test_instance(self):
        self.assertTrue(self.rating.pk, msg=(
            'Rating model should have been created.'))



"""Admin classes for the review app."""
from django.contrib import admin
from django.utils.translation import ugettext_lazy as _

from hvad.admin import TranslatableAdmin

from . import models


class RatingAdmin(admin.ModelAdmin):
    list_display = ['review', 'category', 'value', ]
    raw_id_fields = ['review', ]


class ReviewAdmin(admin.ModelAdmin):
    list_display = ['reviewed_item', 'user', 'language', 'creation_date']


class ReviewExtraInfoAdmin(admin.ModelAdmin):
    list_display = ['type', 'review', 'content_object']


class ReviewCategoryChoiceAdmin(TranslatableAdmin):
    list_display = ['ratingcategory', 'value', 'get_label']

    def get_label(self, obj):
        return obj.label
    get_label.short_description = _('Label')


admin.site.register(models.Rating, RatingAdmin)
admin.site.register(models.RatingCategory, TranslatableAdmin)
admin.site.register(models.Review, ReviewAdmin)
admin.site.register(models.ReviewExtraInfo, ReviewExtraInfoAdmin)
admin.site.register(models.RatingCategoryChoice, ReviewCategoryChoiceAdmin)



"""Views for the review app."""
import importlib

from django.conf import settings
from django.contrib.auth.decorators import login_required
from django.contrib.contenttypes.models import ContentType
from django.core.urlresolvers import reverse
from django.db.models import ObjectDoesNotExist
from django.http import Http404, HttpResponseRedirect
from django.utils import timezone
from django.views.generic import CreateView, DeleteView, DetailView, UpdateView

from .forms import ReviewForm
from .models import Review


#------ MIXINS ------#

class ReviewViewMixin(object):
    model = Review

    def dispatch(self, request, *args, **kwargs):
        # Check, if user needs to be logged in
        if getattr(settings, 'REVIEW_ALLOW_ANONYMOUS', False):
            return super(ReviewViewMixin, self).dispatch(
                request, *args, **kwargs)
        return login_required(super(ReviewViewMixin, self).dispatch)(
            request, *args, **kwargs)

    def get_form_class(self):
        if getattr(settings, 'REVIEW_CUSTOM_FORM', False):
            app_label, class_name = settings.REVIEW_CUSTOM_FORM.rsplit('.', 1)
            try:
                return getattr(importlib.import_module(app_label), class_name,
                               ReviewForm)
            except ImportError:
                pass
        return ReviewForm

    def get_form_kwargs(self, *args, **kwargs):
        kwargs = super(ReviewViewMixin, self).get_form_kwargs(*args, **kwargs)
        kwargs.update({'reviewed_item': self.reviewed_item})
        if self.request.user.is_authenticated():
            kwargs.update({'user': self.request.user})
        return kwargs

    def get_success_url(self):
        success_url = getattr(settings, 'REVIEW_UPDATE_SUCCESS_URL', None)
        if success_url is not None:
            if callable(success_url):
                return success_url(self.object)
            else:
                return reverse(success_url)
        return reverse('review_detail', kwargs={'pk': self.object.pk})


class ReviewUpdateMixin(object):
    """Mixin to provide update functions for a ``Review`` instance."""
    def dispatch(self, request, *args, **kwargs):
        self.kwargs = kwargs
        self.object = self.get_object()
        if not self.object.user or self.object.user != request.user:
            raise Http404
        # Check, if update period is set and has ended
        if getattr(settings, 'REVIEW_UPDATE_PERIOD', False):
            period_end = self.object.creation_date + timezone.timedelta(
                seconds=getattr(settings, 'REVIEW_UPDATE_PERIOD') * 60)
            if timezone.now() > period_end:
                return HttpResponseRedirect(
                    reverse('review_detail', kwargs={'pk': self.object.pk}))
        self.reviewed_item = self.object.reviewed_item
        return super(ReviewUpdateMixin, self).dispatch(
            request, *args, **kwargs)


#------ MODEL VIEWS ------#

class ReviewCreateView(ReviewViewMixin, CreateView):
    """View to create a new ``Review`` instance."""
    def dispatch(self, request, *args, **kwargs):
        # Check, if content type exists
        try:
            self.content_type = ContentType.objects.get(
                model=kwargs.get('content_type'))
        except ContentType.DoesNotExist:
            raise Http404

        # Check, if reviewed item exists
        try:
            self.reviewed_item = self.content_type.get_object_for_this_type(
                pk=kwargs.get('object_id'))
        except ObjectDoesNotExist:
            raise Http404

        # Check for permission
        if request.user.is_authenticated():
            # Check, if user has already reviewed this item
            if getattr(settings, 'REVIEW_AVOID_MULTIPLE_REVIEWS', False):
                try:
                    old_review = Review.objects.filter(
                        user=request.user, content_type=self.content_type,
                        object_id=kwargs.get('object_id'))[0]
                except IndexError:
                    pass
                else:
                    return HttpResponseRedirect(
                        reverse('review_update', kwargs={'pk': old_review.pk}))
            # Check the custom permission function
            has_perm = getattr(settings, 'REVIEW_PERMISSION_FUNCTION', None)
            if (callable(has_perm)
                    and not has_perm(request.user, self.reviewed_item)):
                raise Http404
        return super(ReviewCreateView, self).dispatch(request, *args, **kwargs)


class ReviewDetailView(DetailView):
    """View to display a ``Review`` instance."""
    model = Review


class ReviewUpdateView(ReviewViewMixin, ReviewUpdateMixin, UpdateView):
    """View to update a ``Review`` instance."""
    pass


class ReviewDeleteView(ReviewViewMixin, ReviewUpdateMixin, DeleteView):
    """View to delete a ``Review`` instance."""
    def get_success_url(self):
        if getattr(settings, 'REVIEW_DELETION_SUCCESS_URL', False):
                return reverse(settings.REVIEW_DELETION_SUCCESS_URL)
        return '/'



# flake8: noqa
# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding model 'RatingCategoryChoiceTranslation'
        db.create_table(u'review_ratingcategorychoice_translation', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('label', self.gf('django.db.models.fields.CharField')(max_length=128)),
            ('language_code', self.gf('django.db.models.fields.CharField')(max_length=15, db_index=True)),
            ('master', self.gf('django.db.models.fields.related.ForeignKey')(related_name='translations', null=True, to=orm['review.RatingCategoryChoice'])),
        ))
        db.send_create_signal(u'review', ['RatingCategoryChoiceTranslation'])

        # Adding unique constraint on 'RatingCategoryChoiceTranslation', fields ['language_code', 'master']
        db.create_unique(u'review_ratingcategorychoice_translation', ['language_code', 'master_id'])

        # Adding model 'RatingCategoryChoice'
        db.create_table(u'review_ratingcategorychoice', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('ratingcategory', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['review.RatingCategory'])),
            ('value', self.gf('django.db.models.fields.PositiveIntegerField')(null=True, blank=True)),
        ))
        db.send_create_signal(u'review', ['RatingCategoryChoice'])

        # Adding field 'RatingCategory.counts_for_average'
        db.add_column(u'review_ratingcategory', 'counts_for_average',
                      self.gf('django.db.models.fields.BooleanField')(default=True),
                      keep_default=False)


    def backwards(self, orm):
        # Removing unique constraint on 'RatingCategoryChoiceTranslation', fields ['language_code', 'master']
        db.delete_unique(u'review_ratingcategorychoice_translation', ['language_code', 'master_id'])

        # Deleting model 'RatingCategoryChoiceTranslation'
        db.delete_table(u'review_ratingcategorychoice_translation')

        # Deleting model 'RatingCategoryChoice'
        db.delete_table(u'review_ratingcategorychoice')

        # Deleting field 'RatingCategory.counts_for_average'
        db.delete_column(u'review_ratingcategory', 'counts_for_average')


    models = {
        u'auth.group': {
            'Meta': {'object_name': 'Group'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '80'}),
            'permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': u"orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'})
        },
        u'auth.permission': {
            'Meta': {'ordering': "(u'content_type__app_label', u'content_type__model', u'codename')", 'unique_together': "((u'content_type', u'codename'),)", 'object_name': 'Permission'},
            'codename': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['contenttypes.ContentType']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        u'auth.user': {
            'Meta': {'object_name': 'User'},
            'date_joined': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75', 'blank': 'True'}),
            'first_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'groups': ('django.db.models.fields.related.ManyToManyField', [], {'to': u"orm['auth.Group']", 'symmetrical': 'False', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'is_staff': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_superuser': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'last_login': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'last_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'password': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'user_permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': u"orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'}),
            'username': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '30'})
        },
        u'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        u'generic_positions.objectposition': {
            'Meta': {'object_name': 'ObjectPosition'},
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['contenttypes.ContentType']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'object_id': ('django.db.models.fields.PositiveIntegerField', [], {}),
            'position': ('django.db.models.fields.PositiveIntegerField', [], {'null': 'True', 'blank': 'True'})
        },
        u'review.rating': {
            'Meta': {'ordering': "['category', 'review']", 'object_name': 'Rating'},
            'category': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['review.RatingCategory']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'review': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'ratings'", 'to': u"orm['review.Review']"}),
            'value': ('django.db.models.fields.CharField', [], {'max_length': '20'})
        },
        u'review.ratingcategory': {
            'Meta': {'object_name': 'RatingCategory'},
            'counts_for_average': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'identifier': ('django.db.models.fields.SlugField', [], {'max_length': '32', 'blank': 'True'})
        },
        u'review.ratingcategorychoice': {
            'Meta': {'object_name': 'RatingCategoryChoice'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'ratingcategory': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['review.RatingCategory']"}),
            'value': ('django.db.models.fields.PositiveIntegerField', [], {'null': 'True', 'blank': 'True'})
        },
        u'review.ratingcategorychoicetranslation': {
            'Meta': {'unique_together': "[('language_code', 'master')]", 'object_name': 'RatingCategoryChoiceTranslation', 'db_table': "u'review_ratingcategorychoice_translation'"},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'label': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'language_code': ('django.db.models.fields.CharField', [], {'max_length': '15', 'db_index': 'True'}),
            'master': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'translations'", 'null': 'True', 'to': u"orm['review.RatingCategoryChoice']"})
        },
        u'review.ratingcategorytranslation': {
            'Meta': {'unique_together': "[('language_code', 'master')]", 'object_name': 'RatingCategoryTranslation', 'db_table': "u'review_ratingcategory_translation'"},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'language_code': ('django.db.models.fields.CharField', [], {'max_length': '15', 'db_index': 'True'}),
            'master': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'translations'", 'null': 'True', 'to': u"orm['review.RatingCategory']"}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '256'})
        },
        u'review.review': {
            'Meta': {'ordering': "['-creation_date']", 'object_name': 'Review'},
            'average_rating': ('django.db.models.fields.FloatField', [], {'default': '0'}),
            'content': ('django.db.models.fields.TextField', [], {'max_length': '1024', 'blank': 'True'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['contenttypes.ContentType']"}),
            'creation_date': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'extra_content_type': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'reviews_attached'", 'null': 'True', 'to': u"orm['contenttypes.ContentType']"}),
            'extra_object_id': ('django.db.models.fields.PositiveIntegerField', [], {'null': 'True', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'language': ('django.db.models.fields.CharField', [], {'max_length': '5', 'blank': 'True'}),
            'object_id': ('django.db.models.fields.PositiveIntegerField', [], {}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['auth.User']", 'null': 'True', 'blank': 'True'})
        },
        u'review.reviewextrainfo': {
            'Meta': {'ordering': "['type']", 'object_name': 'ReviewExtraInfo'},
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['contenttypes.ContentType']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'object_id': ('django.db.models.fields.PositiveIntegerField', [], {}),
            'review': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['review.Review']"}),
            'type': ('django.db.models.fields.CharField', [], {'max_length': '256'})
        },
        u'user_media.usermediaimage': {
            'Meta': {'object_name': 'UserMediaImage'},
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['contenttypes.ContentType']", 'null': 'True', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'image': ('django.db.models.fields.files.ImageField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'object_id': ('django.db.models.fields.PositiveIntegerField', [], {'null': 'True', 'blank': 'True'}),
            'thumb_h': ('django.db.models.fields.PositiveIntegerField', [], {'null': 'True', 'blank': 'True'}),
            'thumb_w': ('django.db.models.fields.PositiveIntegerField', [], {'null': 'True', 'blank': 'True'}),
            'thumb_x': ('django.db.models.fields.PositiveIntegerField', [], {'null': 'True', 'blank': 'True'}),
            'thumb_x2': ('django.db.models.fields.PositiveIntegerField', [], {'null': 'True', 'blank': 'True'}),
            'thumb_y': ('django.db.models.fields.PositiveIntegerField', [], {'null': 'True', 'blank': 'True'}),
            'thumb_y2': ('django.db.models.fields.PositiveIntegerField', [], {'null': 'True', 'blank': 'True'}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['auth.User']"})
        }
    }

    complete_apps = ['review']
